@@ -22,19 +22,17 @@ extern gpfunc listfunc[];
 
 gpdesc *newdesc(int nb)
 {
-  gpdesc *gd;
-  gd=malloc(sizeof(*gd));
+  gpdesc *gd = (gpdesc *) malloc(sizeof(*gd));
   gd->nb=nb;
-  gd->a=nb?calloc(nb,sizeof(*(gd->a))):NULL;
+  gd->a=nb?(gpdescarg *)calloc(nb,sizeof(*(gd->a))):NULL;
   return gd;
 }
 
 gpwrap *newwrap(int nb)
 {
-  gpwrap *gw;
-  gw=malloc(sizeof(*gw));
+  gpwrap *gw = (gpwrap *) malloc(sizeof(*gw));
   gw->nb=nb;
-  gw->w=nb?calloc(nb,sizeof(*(gw->w))):NULL;
+  gw->w=nb?(int *)calloc(nb,sizeof(*(gw->w))):NULL;
   return gw;
 }
 
@@ -86,6 +84,7 @@ strtoargs(char *s)
   case '"':
     aa.t=AAstring;
     aa.str=xstrndup(s+1,strlen(s)-2);
+    return aa;
     break;
   case '@':
     aa.t=AAmulti;
@@ -166,8 +165,7 @@ void initdesc(char *descfile)
     die(err_desc,"Cannot find description file %s",descfile);
   while(!feof(dfile))
   {
-    int i;
-    int nb,ndesc,nf;
+    int i, nb, nf;
     gpfunc *func;
     gpdesc *gd;
     if (!fgets(buf,BUFFER_SIZE,dfile))
@@ -179,63 +177,84 @@ void initdesc(char *descfile)
     nb=readnumber(dfile,buf,BUFFER_SIZE);/*number of description*/
     if (nb<0)
       die(err_desc,"Bad description file %s, func %s",descfile,func->gpname);
-    gd=newdesc(nb); ndesc=0;
     for(i=0;i<nb;i++)
     {
-      int j,nargs;
-      char *data;
-      readentry(dfile,buf,BUFFER_SIZE);
-      data=strdup(buf);
-      nargs=readnumber(dfile,buf,BUFFER_SIZE);
-      if (nargs>=0)
-      {  /* This is a description*/
-        gpdescarg *da=gd->a+(ndesc++);
-        da->cname=data;
-        da->nargs=nargs;
-        if (nargs)
-          da->args=calloc(nargs,sizeof(*da->args));
-        else
-          da->args=NULL;
-        for(j=0;j<nargs;j++)
-        {
-          readentry(dfile,buf,BUFFER_SIZE);
-          da->args[j]=strtoargs(buf);
-        }
-        da->type=readtypemode(dfile,buf,BUFFER_SIZE,&da->mode);
-      }
-      else
+      int j,nargs,type,ndesc;
+      type=readnumber(dfile,buf,BUFFER_SIZE);
+      switch(type)
       {
-        switch(-nargs)
-        {
+        case 0: case 3:
+          /* This is a description or an iterator*/
+          ndesc=readnumber(dfile,buf,BUFFER_SIZE);
+          if (ndesc)
+          {
+            gd = newdesc(ndesc);
+            if (type==0) func->dsc=gd;
+            else         func->iter=gd;
+          }
+          for(j=0; j<ndesc; j++)
+          {
+            int k;
+            gpdescarg *da=gd->a+j;
+            if (type==0)
+            {
+              readentry(dfile,buf,BUFFER_SIZE);
+              da->cname=strdup(buf);
+            }
+            else
+            {
+              gpiterator *iter = &(da->iter);
+              int niter=readnumber(dfile,buf,BUFFER_SIZE);
+              if (niter!=3) die(err_desc,"unsupported iterator");
+              readentry(dfile,buf,BUFFER_SIZE);
+              iter->type=strtotype(buf);
+              readentry(dfile,buf,BUFFER_SIZE);
+              iter->init=getfunc(buf); func = lfunc+nf;
+              readentry(dfile,buf,BUFFER_SIZE);
+              iter->next=getfunc(buf); func = lfunc+nf;
+            }
+            nargs=readnumber(dfile,buf,BUFFER_SIZE);
+            da->nargs=nargs;
+            if (nargs)
+              da->args=(descargatom *) calloc(nargs,sizeof(*da->args));
+            else
+              da->args=NULL;
+            for(k=0; k<nargs;k++)
+            {
+              readentry(dfile,buf,BUFFER_SIZE);
+              da->args[k]=strtoargs(buf);
+            }
+            da->type=readtypemode(dfile,buf,BUFFER_SIZE,&da->mode);
+          }
+          break;
         case 1: /*This is a prototype*/
-          func->proto.cname=data;
+          readentry(dfile,buf,BUFFER_SIZE);
+          func->proto.cname=strdup(buf);
           readentry(dfile,buf,BUFFER_SIZE);
           func->proto.code=strdup(buf);
           functype(*func)=readtypemode(dfile,buf,BUFFER_SIZE,&funcmode(*func));
           break;
         case 2: /*This is a wrapper*/
-          nargs = atol(data);
-          free(data);
+          nargs=readnumber(dfile,buf,BUFFER_SIZE);
           func->wrap = newwrap(nargs);
           for(j=0;j<nargs;j++)
           {
+            int nw;
             readentry(dfile,buf,BUFFER_SIZE);
-            func->wrap->w[j]=*buf?(buf[1]?getfunc(buf):-1):-2;
+            nw = getfunc(buf); func = lfunc+nf; /* getfunc can realloc lfunc */
+            func->wrap->w[j]=*buf?(buf[1]?nw:-1):-2;
           }
           break;
         default:
-          die(err_desc,"Unknown description type %d in %s",nargs,descfile);
-        }
+          die(err_desc,"Unknown description type %d in %s",type,descfile);
       }
     }
-    gd->nb=ndesc;
-    func->dsc=ndesc?gd:NULL;
   }
   fclose(dfile);
 }
 
 int
-descrulescore(int nb, int *args, gpdescarg *ga, int * const psc,int * const pesc, gpfunc *gp)
+descrulescore(int nb, int *args, gpdescarg *ga, int * const psc,int * const pesc)
 {
   int sc=0, esc=0;
   int i,j;
@@ -320,9 +339,11 @@ descrulescore(int nb, int *args, gpdescarg *ga, int * const psc,int * const pesc
       }
       return 1;
     case AAlvalue:
-      if (t==da[j].type && getlvalue(arg)>=0)
+      if (getlvalue(arg)<0)
+        return 1;
+      if (t==da[j].type)
         esc++;
-      else
+      else if (ctype[t]!=ctype[da[j].type] || !is_subtype(t,da[j].type))
         return 1;
       break;
     case AAmulti:
@@ -340,16 +361,15 @@ descrulescore(int nb, int *args, gpdescarg *ga, int * const psc,int * const pesc
   return 0;
 }
 
-gpdescarg *descfindrules(int nb, int *arg, gpfunc *gp)
+gpdescarg *descfindrulesdsc(int nb, int *arg, gpdesc *dsc)
 {
   int i;
   int best=-1,score=-1,escore=-1;
-  gpdesc *dsc=gp->dsc;
   gpdescarg *ga=dsc->a;
   for(i=0;i<dsc->nb;i++)
   {
     int sc=0, esc=0;
-    if (descrulescore(nb,arg,ga+i,&sc,&esc,gp))
+    if (descrulescore(nb,arg,ga+i,&sc,&esc))
       continue;
     if (best==-1 || sc<score || (sc==score &&  esc>escore ))
     {
@@ -361,6 +381,12 @@ gpdescarg *descfindrules(int nb, int *arg, gpfunc *gp)
   return (best==-1)?NULL:ga+best;
 }
 
+
+gpdescarg *descfindrules(int nb, int *arg, gpfunc *gp)
+{
+  return descfindrulesdsc(nb, arg, gp->dsc);
+}
+
 gpdescarg *descfindrules1(int arg, int nf)
 {
   return descfindrules(1, &arg, lfunc+nf);
@@ -369,7 +395,7 @@ gpdescarg *descfindrules1(int arg, int nf)
 int gentypefuncdesc(int n, gpfunc *gp)
 {
   int arg[STACKSZ];
-  int nb;
+  int i, nb;
   int y=tree[n].y;
   gpdescarg *rule;
   if ( y!=-1 )
@@ -381,11 +407,18 @@ int gentypefuncdesc(int n, gpfunc *gp)
   rule=descfindrules(nb,arg,gp);
   if (!rule)
     return Gnotype;
+  for(i=0;i<rule->nargs;i++)
+    if (rule->args[i].t==AAlvalue)
+    {
+      int ta=tree[arg[i]].t, ts=rule->args[i].type;
+      if (!is_subtype(ts,ta))
+        warning(n,"Assignement to a less precise type: %s<-%s",GPname(ta),GPname(ts));
+    }
   tree[n].m|=rule->mode;
   return rule->type;
 }
 
-enum {FBparens, FBlong, FBstdref} flagbit;
+enum FBenum {FBparens, FBstdref} flagbit;
 
 struct rpn_data
 {
@@ -422,17 +455,14 @@ static descargatom* get_atom(struct rpn_data *d, int n)
 
 static void cast_flag(struct rpn_data *d, int n, int t)
 {
-  if (d->flag&(1<<FBlong))
-    gencastl(d->fout,n,t,d->flag&(1<<FBparens));
-  else
-    gencastf(d->fout,n,t,d->flag&(1<<FBparens));
+  gencastf(d->fout,n,t,d->flag&(1<<FBparens));
 }
 
 #define RPN(f) void (f)(struct rpn_data *d, int *stk)
 
 struct rpn_func
 {
-  char *name;
+  const char *name;
   int arity;
   RPN(*function);
 };
@@ -440,6 +470,7 @@ struct rpn_func
 #define pop(n)  d->sp-=(n)
 #define push(n) d->sp+=(n)
 #define LVL(n)  (stk[d->sp-1-(n)])
+#define NOLVL   ((void)stk)
 
 RPN(rpn_add) {LVL(1)+=LVL(0); pop(1);}
 RPN(rpn_sub) {LVL(1)-=LVL(0); pop(1);}
@@ -456,9 +487,8 @@ RPN(rpn_not) {LVL(0)=!LVL(0);}
 
 RPN(rpn_nbarg)  {push(1);LVL(0)=d->nbarg;}
 
-RPN(rpn_parens) {d->flag|=1<<FBparens;}
-RPN(rpn_long)   {d->flag|=1<<FBlong;}
-RPN(rpn_stdref)   {d->flag|=1<<FBstdref;}
+RPN(rpn_parens) {NOLVL;d->flag|=1<<FBparens;}
+RPN(rpn_stdref) {NOLVL;d->flag|=1<<FBstdref;}
 
 RPN(rpn_str_format) { genpercent(d->fout, get_str(d, LVL(0))); pop(1); }
 RPN(rpn_str_raw)    { fputs(entryname(get_str(d, LVL(0))),d->fout); pop(1); }
@@ -618,7 +648,7 @@ const char *gencoderpn(FILE *fout, const char *p, gpdescarg *rule, int nbarg, in
     {"mul",2,rpn_mul},{"div",2,rpn_div},{"mod",2,rpn_mod},
     {"and",2,rpn_and},{"or",2,rpn_or},{"xor",2,rpn_xor},{"not",1,rpn_not},
     {"value",1,rpn_value},{"type",1,rpn_type},{"nbarg",0,rpn_nbarg},
-    {"parens",0,rpn_parens},{"long",0,rpn_long},{"stdref",0,rpn_stdref},
+    {"parens",0,rpn_parens},{"stdref",0,rpn_stdref},
     {"str_format",1,rpn_str_format},{"str_raw",1,rpn_str_raw},
     {"code",1,rpn_code},{"cast",2,rpn_cast},
     {"format_string",1,rpn_format_string}, {"format_args",1,rpn_format_args},
@@ -696,7 +726,7 @@ void gencodedesc(FILE *fout, int nb, int *arg, gpdescarg *rule, int nerr, int nf
         break;
       case '"':
         {
-          char *s=memccpy(buf,p+1,'"',STACKSZ-1);
+          char *s=(char*) memccpy(buf,p+1,'"',STACKSZ-1);
           if (!s)
             die(nerr,"Unfinished \" in description");
           *(s-1)=0;
