@@ -59,25 +59,6 @@ void gencastf(FILE *fout, int n, int nt, int f)
   die(-1,"No conversion method defined for %s->%s",GPname(t),GPname(nt));
 }
 
-void gencastl(FILE *fout, int n, int nt, int f)
-{
-  int m=tree[n].m;
-  if ((m&(1<<Mlong)) && ctype[nt]==Vgen && ctype[tree[n].t]==Vgen)
-  {
-    if (f)
-      genparensg(fout,n);
-    else
-      gencodeg(fout,n);
-  }
-  else
-  {
-    if (f) fprintf(fout,"(");
-    fprintf(fout,"(long) ");
-    gencastf(fout,n,nt,0);
-    if (f) fprintf(fout,")");
-  }
-}
-
 void gencast(FILE *fout, int n, int nt)
 {
   if (nt<s_GPtype.n)
