@@ -31,11 +31,12 @@ static int newfunc(const char *gpname)
   f->proto.cname=gpname;
   f->proto.code=NULL;
   f->proto.origin=NULL;
-  f->proto.help=NULL;
+  f->proto.help=-1;
   f->node=newnode(Ffun,nf,-1);
   functype(*f)=Gempty;
   funcmode(*f)=0;
   f->spec=GPpari;
+  f->iter=NULL;
   f->wrap=NULL;
   f->dsc=NULL;
   f->user=NULL;
@@ -59,12 +60,12 @@ int newuserfunc(const char *gpname)
   f->spec=GPuser;
   f->proto.origin=namelib;
   uf=f->user=(userfunc*) malloc(sizeof(*f->user));
-  stack_init(&uf->v,sizeof(*uf->var),(void *)&uf->var);
-  stack_init(&uf->g,sizeof(*uf->gcvar),(void *)&uf->gcvar);
+  stack_init(&uf->v,sizeof(*uf->var),(void **)&uf->var);
+  stack_init(&uf->g,sizeof(*uf->gcvar),(void **)&uf->gcvar);
   return nf;
 }
 
-int findfunction_len(const char *s, int n)
+int findfunction_len(const char *s, size_t n)
 {
   int i;
   for(i=0; i<s_func.n;i++)
@@ -101,13 +102,22 @@ int findfuncdescopt(const char *s)
   return n;
 }
 
-int findfunctype(char *s)
+int findfunctype(const char *s)
 {
   int nf=findfuncdesc(s);
   gpdescarg *da=descfindrules(0,NULL,lfunc+nf);
   return da->type;
 }
 
+int findfunctypeopt(const char *s)
+{
+  gpdescarg *da;
+  int nf=findfuncdescopt(s);
+  if (nf<0) return -1;
+  da=descfindrules(0,NULL,lfunc+nf);
+  return da->type;
+}
+
 int getfunc(const char *gpname)
 {
   int r=findfunction(gpname);
@@ -141,15 +151,16 @@ int genarg(int nerr, FILE *fout, char c, int n, PPproto pr)
     gencast(fout,n,Ggen);
     return 1;
   case 'L':
+  case 'P':
     gencast(fout,n,Gsmall);
     return 1;
   case '&':
     if (tree[n].f!=Frefarg && pr == PPdefault)
       die(nerr,"Missing & for reference");
+  case '*': /* Fall through */
     fprintf(fout,"&");
     gencode(fout,n);
     return 1;
-  case '*':
   case 'W':
     gencode(fout,n);
     return 1;
@@ -296,10 +307,10 @@ void genuserfunc(FILE *fout, int n, int nf)
     firstarg=1;
     if (j<nb && arg[j]>=0 && arg[j]!=GNOARG)
       gencast(fout,arg[j],t);
-    else if (v->initval<0)
-      die(n,"a mandatory argument is missing");
     else if (v->flag&(1<<Cdefmarker))
       gencodenoarg(fout,t,n);
+    else if (v->initval<0)
+      die(n,"a mandatory argument is missing");
     else
       gencast(fout,v->initval,t);
   }
@@ -385,7 +396,7 @@ void genvarproto(FILE *fout, int n, int nerr)
     gencode(fout,n);
 }
 
-void genfuncproto(FILE *fout, int nf, char *sep, int nerr)
+void genfuncproto(FILE *fout, int nf, const char *sep, int nerr)
 {
   gpfunc *gp=lfunc+nf;
   int n=gp->node;
@@ -543,7 +554,6 @@ void genprototype(FILE *fout, int nf, int kb)
         printnode(fout,v->initval);
         fprintf(fout,"*/");
       }
-
     }
   }
   s_ctx.n=savc;
@@ -652,7 +662,7 @@ void genwrapper(FILE *fout, int nf, int wrap)
   {
     genfuncbydesc1(fout,vres->node,FC_decl_base,nerr);
     fprintf(fout," ");
-    if (genfuncbydesc1(fout,vres->node,FC_decl_ext,nerr));
+    if (genfuncbydesc1(fout,vres->node,FC_decl_ext,nerr))
       gencode(fout,vres->node);
     fprintf(fout," = ");
   }
@@ -727,7 +737,7 @@ void gendefblock(FILE *fout, int n)
   int b=tree[n].x;
   int seq=tree[n].y;
   int i;
-  int t,m;
+  int m;
   int savc;
   context *fc=block+b;
   savc=s_ctx.n;
@@ -744,24 +754,20 @@ void gendefblock(FILE *fout, int n)
     }
   }
   pushctx(fc);
-  /*some shortcut*/
-  t=tree[n].t;
   m=tree[n].m;
   if(!(m&(1<<Mbrace)))
   {
-    genindent(fout);
-    fprintf(fout,"{\n");
+    genindentline(fout,"{\n");
     indent++;
   }
   gendecvar(fout,fc,n);
   genindentseq(fout,seq);
   gencode(fout,seq);
-  gensemicomma(fout,seq);
+  gensemicolon(fout,seq);
   if(!(m&(1<<Mbrace)))
   {
     indent--;
-    genindent(fout);
-    fprintf(fout,"}\n");
+    genendbrace(fout,n);
   }
   s_ctx.n=savc;
 }
