@@ -128,7 +128,6 @@ void gentypedeffunc(int n)
   int savcf;
   int tf;
   gpfunc *gp;
-  context *fc;
 
   /*save current function*/
   savcf=currfunc;
@@ -139,7 +138,6 @@ void gentypedeffunc(int n)
   gentype(seq);
   /*gentype may change functype(*gp)*/
   tf=functype(*gp);
-  fc=block+gp->user->bl;
   if (tf==Gnegbool) tf=functype(*gp)=Gbool;
   if (tf==Glg) tf=functype(*gp)=Gsmall;
   if (tf==Gstr) tf=functype(*gp)=Ggenstr;
@@ -232,7 +230,7 @@ int gentype(int n)
     tree[n].t=Gnotype;
     tree[n].m=(mx|my)&MODHERIT;
     break;
-  case Faffect:
+  case Fassign:
     {
       int z,lx;
       ctxvar *v;
@@ -267,13 +265,13 @@ int gentype(int n)
         }
       }
       if ((tree[z].f!=Fentry || (v->flag&(1<<Ccompo)))
-       && (tree[y].f==Fentry || tree[y].f==Faffect))
+       && (tree[y].f==Fentry || tree[y].f==Fassign))
         tree[y].m|=(1<<Mcopy);
       if (v->flag&(1<<Ccompo))
         tree[n].m|=(1<<Mcopy);
       if (tx!=Gvoid)
         tree[n].m|=(1<<Msidef);
-      tree[n].m|=(tree[z].m&(1<<Mlong))|(1<<Mparens);
+      tree[n].m|=1<<Mparens;
     }
     break;
   case Fconst:
@@ -319,7 +317,7 @@ int gentype(int n)
   case Frefarg:
     gentype(x);
     tree[n].t=tree[x].t;
-    tree[n].m=(tree[x].m&~(1<<Mlong));
+    tree[n].m=tree[x].m;
     break;
   case Fentry:
     c=getvar(n);
@@ -351,7 +349,7 @@ int gentype(int n)
         gentype(y);
         tree[n].t=functype(*gp);
         my=(y==-1)?0:tree[y].m;
-        tree[n].m=(funcmode(*gp)&(~(1<<Msemicomma)))|(my&MODHERIT);
+        tree[n].m=(funcmode(*gp)&(~(1<<Msemicolon)))|(my&MODHERIT);
       }
     }
     else
