@@ -21,7 +21,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 void gerepilelist(stack *s, stack *g, int savb)
 {
   affnode *an=(affnode *) *stack_base(s);
-  int **gcvar=(int **) stack_base(g);
   int i;
   for(i=0;i<s->n;i++)
   {
@@ -31,7 +30,9 @@ void gerepilelist(stack *s, stack *g, int savb)
     if (ctype[t]!=Vgen || ((v->flag&(1<<Cconst)) && v->val!=-1))
       continue;
     /*If i is affected or referenced*/
-    if (an[i].f!=AFaccess && an[i].f!=AFhide && an[i].f!=AFclone)
+    if (an[i].f==AFclone)
+      stack_remove_int(g,v->node);
+    else if (an[i].f!=AFaccess && an[i].f!=AFhide)
     {
       int j;
       /*search the first occurence of i*/
@@ -40,26 +41,22 @@ void gerepilelist(stack *s, stack *g, int savb)
           break;
       /*if the variable is local and the first occurence
         is an affectation, do nothing */
-      if (an[j].idx>=savb &&  an[j].f==AFaffect)
+      if (an[j].idx>=savb &&  an[j].f==AFassign)
         continue;
 
       /*else check if it is not already in the list*/
-      for(j=0;j<g->n;j++)
-        if ((*gcvar)[j]==idx)
-          break;
-      /*if it is already, do nothing*/
-      if (j<g->n) continue;
-      /*add to list*/
-      j=stack_new(g);
-      (*gcvar)[j]=idx;
+      if (stack_has_int(g,v->node))
+        continue;
+      stack_push_int(g,v->node);
     }
   }
+  stack_int_sort(g);
   if (debug)
   {
     int i;
     fprintf(stderr,"/*");
     for(i=0;i<s->n;i++)
-      fprintf(stderr,"%s%s ",varstr(ctxstack[an[i].idx]),an[i].f==AFaffectcompo?"=[]":an[i].f==AFaffect?"=":"");
+      fprintf(stderr,"%s%s ",varstr(ctxstack[an[i].idx]),an[i].f==AFassigncompo?"=[]":an[i].f==AFassign?"=":"");
     fprintf(stderr,"*/\n");
   }
 }
@@ -74,14 +71,14 @@ void pilelistvar(int idx)
   if (v->initval>=0)
   {
     pilelist(v->initval);
-    newaff(AFaffect,idx);
+    newaff(AFassign,idx);
   }
 }
 
 void pilelist(int n)
 {
   int x,y;
-  int savc,savb;
+  int savc;
   context *bl;
   if (n<0)
     return;
@@ -89,7 +86,7 @@ void pilelist(int n)
   y=tree[n].y;
   switch(tree[n].f)
   {
-  case Faffect:
+  case Fassign:
     if (tree[x].f!=Fentry)
       pilelist(x);
     pilelist(y);
@@ -137,7 +134,6 @@ void pilelist(int n)
     savc=s_ctx.n;
     bl=block+tree[n].x;
     pushctx(bl);
-    savb=s_ctx.n;
     pilelist(y);
     if (bl->gc&(1<<GCneeded))
     {
@@ -146,19 +142,15 @@ void pilelist(int n)
       {
         int vret=getvar(bl->egc);
         ctxvar *v=ctxstack+vret;
-        if (ctype[vartype(*v)]==Vgen)
-        {
-          int **gcvar=(int **) stack_base(&bl->g);
-          int j=stack_new(&bl->g);
-          (*gcvar)[j]=vret;
-        }
+        if (ctype[vartype(*v)]==Vgen && !stack_has_int(&bl->g,vret))
+          stack_push_int(&bl->g,v->node);
       }
     }
     copyctx(savc,bl);
     s_ctx.n=savc;
     break;
   default:
-    if (tree[n].f>=FneedENTRY || tree[n].f<0)
+    if (tree[n].f>=FneedENTRY)
       die(n,"Incorrect node %s in pilelist",funcname(tree[n].f));
     pilelist(x);
     pilelist(y);
@@ -175,7 +167,7 @@ void pileclean(int n)
   y=tree[n].y;
   switch(tree[n].f)
   {
-  case Faffect:
+  case Fassign:
     pileclean(x);
     pileclean(y);
     break;
@@ -212,53 +204,17 @@ void pileclean(int n)
     if (bl->gc&(1<<GCglobal))
     {
       userfunc *ufunc=lfunc[currfunc].user;
-      int i,j;
-      int *var;
-      stack v;
-      stack_init(&v,sizeof(*var),(void *)&var);
-      for(i=0,j=0;i<bl->g.n || j<ufunc->g.n;)
-      {
-        if (j==ufunc->g.n
-            || (i<bl->g.n && bl->gcvar[i]<ufunc->gcvar[j]))
-        {
-          int k=stack_new(&v);
-          var[k]=bl->gcvar[i++];
-        }
-        else
-        {
-          int k=stack_new(&v);
-          var[k]=ufunc->gcvar[j];
-          if (i<bl->g.n  && bl->gcvar[i]==ufunc->gcvar[j])
-            i++;
-          j++;
-        }
-      }
-      stack_replace(&bl->g,&v);
+      stack_int_merge(&bl->g,&ufunc->g);
       if (bl->g.n!=1)
         bl->gc&=~(1<<GCupto);
     }
     if ((bl->gc&(1<<GCneeded)) && (!bl->g.n || (bl->gc&(1<<GCupto))))
     {
-      if (bl->gc&(1<<GCglobal))
-      {
-        if (FC_gerepileall<0)
-        {
-          int v=getvarbyname("gptr");
-          if (v>=savc && v<savb)
-            vartype(ctxstack[v])=Gvoid;
-        }
-      }
-      else
+      if (!(bl->gc&(1<<GCglobal)))
       {
         int w=getvarbyname("st_lim");
         if (w>=savc && w<savb)
           vartype(ctxstack[w])=Gvoid;
-        if (FC_gerepileall<0)
-        {
-          int v=getvarbyname("bptr");
-          if (v>=savc && v<savb)
-            vartype(ctxstack[v])=Gvoid;
-        }
       }
     }
     if ((bl->gc&(1<<GCreturn)) &&
@@ -267,9 +223,10 @@ void pileclean(int n)
       tree[n].m|=(1<<Mbrace);
     copyctx(savc,bl);
     s_ctx.n=savc;
+    s_aff.n=sava;
     break;
   default:
-    if (tree[n].f>=FneedENTRY || tree[n].f<0)
+    if (tree[n].f>=FneedENTRY)
       die(n,"Incorrect node %s in pileclean",funcname(tree[n].f));
     pileclean(x);
     pileclean(y);
