@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000-2010  The PARI group.
+Copyright (C) 2000-2013  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -35,6 +35,13 @@ int optcleanvar=0;/*used in genbrace to suppress {} optimization*/
 int indentlevel=2;
 int do_warning=1;
 char *optprefix,*optsuffix;
+int debug;
+const char *nameparse, *namelib;
+int lastpass;
+int autogc;
+int warn;
+int safecoeff;
+
 void printlistfunc(FILE *fout);
 void outputtype(FILE *fout);
 void cleancode(int n, int p);
@@ -76,18 +83,18 @@ void
 init_compiler(void)
 {
   int n;
-  stack_init(&s_node,sizeof(*tree),(void *)&tree);
-  stack_init(&s_value,sizeof(*value),(void *)&value);
-  stack_init(&s_func,sizeof(*lfunc),(void *)&lfunc);
-  stack_init(&s_ctx,sizeof(*ctxstack),(void *)&ctxstack);
-  stack_init(&s_aff,sizeof(*affstack),(void *)&affstack);
-  stack_init(&s_bloc,sizeof(*block),(void *)&block);
-  stack_init(&s_comment,sizeof(*com),(void *)&com);
-  stack_init(&s_errors,sizeof(*errors),(void *)&errors);
-  stack_init(&s_Ctype,sizeof(*Ctype),(void *)&Ctype);
-  stack_init(&s_GPtype,sizeof(*GPtype),(void *)&GPtype);
-  stack_init(&s_Mmode,sizeof(*Mmode),(void *)&Mmode);
-  stack_init(&s_label,sizeof(*label),(void *)&label);
+  stack_init(&s_node,sizeof(*tree),(void **)&tree);
+  stack_init(&s_value,sizeof(*value),(void **)&value);
+  stack_init(&s_func,sizeof(*lfunc),(void **)&lfunc);
+  stack_init(&s_ctx,sizeof(*ctxstack),(void **)&ctxstack);
+  stack_init(&s_aff,sizeof(*affstack),(void **)&affstack);
+  stack_init(&s_bloc,sizeof(*block),(void **)&block);
+  stack_init(&s_comment,sizeof(*com),(void **)&com);
+  stack_init(&s_errors,sizeof(*errors),(void **)&errors);
+  stack_init(&s_Ctype,sizeof(*Ctype),(void **)&Ctype);
+  stack_init(&s_GPtype,sizeof(*GPtype),(void **)&GPtype);
+  stack_init(&s_Mmode,sizeof(*Mmode),(void **)&Mmode);
+  stack_init(&s_label,sizeof(*label),(void **)&label);
    /*Node 0 should be a (Gvoid)Fnoarg*/
   n=newnode(Fnoarg,-1,-1); tree[n].t=Gvoid;
    /*Node 1 should be a (Gnotype)Fnoarg*/
@@ -114,12 +121,17 @@ init_stdfunc(void)
   FC_copy=findfuncdesc("copy");
   FC_avma=findfuncdesc("_avma");
   Gpari_sp=findfunctype("_avma");
-  Gbptr=findfunctype("_diffptr");
   FC_low_stack_lim=findfuncdesc("_low_stack_lim");
-  /* GP 2.1 has no gerepileall but is supported*/
-  FC_gerepileall=findfuncdescopt("_gerepileall");
-  Ggptr = (FC_gerepileall<0)?strtotype("gptr"):Gnotype;
-  FC_forprime_next=findfuncdescopt("_forprime_next");
+  FC_gerepileall=findfuncdesc("_gerepileall");
+  FC_forprime_init=findfuncdescopt("_forprime_init");
+  if (FC_forprime_init>=0)
+    Gforprime=strtotype("forprime");
+  else
+  {
+    Gforprime=findfunctype("_diffptr");
+    FC_forprime_next=findfuncdesc("_forprime_next");
+  }
+  Gerror=findfunctypeopt("_iferr_error");
   FC_matrixrow=findfuncdescopt("_[_,]");
   FC_const_real=findfuncdescopt("_const_real");
   FC_call=findfuncdescopt("_(_)");
@@ -127,6 +139,11 @@ init_stdfunc(void)
   FC_proto_max_args=findfuncdescopt("_proto_max_args");
   max_args = FC_proto_max_args<0 ? 8 :
              lfunc[FC_proto_max_args].dsc->a[0].args[0].misc;
+  if (safecoeff && findfuncdescopt("_safecoeff")<0)
+  {
+    warning(-1,"gp2c option -C not supported by this PARI version");
+    safecoeff=0;
+  }
 }
 
 void
@@ -145,17 +162,19 @@ init_typedef(void)
 }
 
 void
-compile(FILE *fin, FILE *fout, char *nom)
+compile(FILE *fin, FILE *fout, const char *nom)
 {
-  char *descfile;
+  const char *descfile;
+  char *tmplib;
   int startnode;
   int status;
   int i;
   init_compiler();
   linecount=1;
   nameparse=nom;
-  namelib=calloc(strlen(nameparse)+6,sizeof(*nameparse));
-  sprintf(namelib,"%s%s.so",nameparse[0]=='/'?"":"./",nameparse);
+  tmplib=(char*)calloc(strlen(nameparse)+6,sizeof(*nameparse));
+  sprintf(tmplib,"%s%s.so",nameparse[0]=='/'?"":"./",nameparse);
+  namelib=tmplib;
   yyin=fin;
   currfunc=-1;
   initoperators();
@@ -275,7 +294,7 @@ compile(FILE *fin, FILE *fout, char *nom)
 void version(void)
 {
   printf("GP to C compiler version %s \n   targeted at PARI/GP %s\n",VERSION,PARI_VERSION);
-  printf("Copyright 2000-2010 The PARI Group\n");
+  printf("Copyright 2000-2013 The PARI Group\n");
   printf("GP2C is free software, covered by the GNU General Public License, and \n\
 you are welcome to change it and/or distribute copies of it under \n\
 certain conditions.  There is absolutely no warranty for GP2C.\n");
@@ -310,20 +329,21 @@ debugging options: \n\
 file.gp: file to be processed, default to stdin. \n\
 The generated C code is output to stdout unless the -o option is used \n\
  \n\
-You can automated the process of compiling with \n\
-the script gp2c-run. \n\
+See the script gp2c-run for an automated compilation process. \n\
 ",s);
 }
 int main(int argc, char **argv)
 {
   int c;
-  FILE *fin;
+  FILE *fin=stdin;
   FILE *fout=stdout;
+  const char *outfile = NULL, *infile = "stdin";
   autogc=0;
+  safecoeff=0;
   warn=0;
   optprefix=NULL;
   optsuffix=NULL;
-  while((c=getopt(argc,argv,"gi:hflo:p:s:StvydTGVW"))!=-1)
+  while((c=getopt(argc,argv,"gi:hflo:p:s:StvydCGTVW"))!=-1)
   {
     switch(c)
     {
@@ -342,7 +362,7 @@ int main(int argc, char **argv)
       break;
     case 'l':
       {
-        char *descfile;
+        const char *descfile;
         init_compiler();
         if (!(descfile=getenv("GP2C_FUNC_DSC")))
           descfile=FUNCDSC_PATH;
@@ -352,7 +372,7 @@ int main(int argc, char **argv)
         exit(0);
       }
     case 'o':
-      fout=fopen(optarg,"w");
+      outfile=strdup(optarg);
       break;
     case 'p':
       optprefix=strdup(optarg);
@@ -385,6 +405,9 @@ int main(int argc, char **argv)
     case 'V':
       optcleanvar=1-optcleanvar;
       break;
+    case 'C':
+      safecoeff=1-safecoeff;
+      break;
     case 'W':
       warn=1-warn;
       break;
@@ -399,20 +422,22 @@ int main(int argc, char **argv)
     usage(stderr,argv[0]);
     exit(1);
   }
-  if (argc==optind)
-    compile(stdin,fout,"stdin");
-  else
+  if (argc!=optind)
   {
-    char *nom=argv[optind];
-    char *nomf=nom;
-    if ((fin=fopen(nomf,"r"))==NULL)
+    infile = argv[optind];
+    if ((fin=fopen(infile,"r"))==NULL)
     {
       perror(argv[0]);
       exit(errno);
     }
-    compile(fin,fout,nom);
-    fclose(fin);
   }
+  if (outfile && ((fout=fopen(outfile,"w"))==NULL))
+  {
+    perror(argv[0]);
+    exit(errno);
+  }
+  compile(fin,fout,infile);
+  if (fin!=stdin) fclose(fin);
   if (fout!=stdout) fclose(fout);
   return 0;
 }
