@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2000  The PARI group.
+Copyright (C) 2000-2003  The PARI group.
 
 This file is part of the GP2C package.
 
@@ -17,37 +17,63 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 #include <stdlib.h>
 #include <stdarg.h>
 #include "header.h"
-char *nameparse;
-void die(int n, char *format, ...)
+int linecount;
+static void
+msgprefix(int n)
 {
-  va_list ap;
-  va_start(ap, format);
   if (n>0)
-    fprintf(stderr,"%s:%d:",nameparse,tree[n].lineno);
+    fprintf(stderr,"%s:%d: ",nameparse,tree[n].lineno);
   else
-    fprintf(stderr,"%s:%s:",nameparse,currfunc==-1?"toplevel":lfunc[currfunc].gpname); 
-  vfprintf(stderr,format,ap);
-  fprintf(stderr,"\n"); 
-  printnode(stderr,n);
-  fprintf(stderr,"\n"); 
-  exit(1);
+  {
+    switch(n)
+    {
+    case err_func:
+      fprintf(stderr,"%s:%s: ",nameparse,currfunc==-1?"toplevel":lfunc[currfunc].gpname);
+      break;
+    case err_parse:
+      fprintf(stderr,"%s:%d: ",nameparse,linecount);
+      break;
+    case err_desc:
+      fprintf(stderr,"func.dsc:<loading>: ");
+      break;
+    }
+  }
 }
-void warning(int n, char *format, ...)
+
+void die(int n, const char *format, ...)
 {
   va_list ap;
+  static int dead;
   va_start(ap, format);
-  if (n>0)
-    fprintf(stderr,"Warning:%s:%d:",nameparse,tree[n].lineno);
-  else
-    fprintf(stderr,"Warning:%s:%s:",nameparse,currfunc==-1?"toplevel":lfunc[currfunc].gpname); 
+  fprintf(stderr,"Error:");
+  msgprefix(n);
   vfprintf(stderr,format,ap);
-  fprintf(stderr,"\n"); 
+  if (dead++) exit(1);
+  fprintf(stderr,"\n");
   printnode(stderr,n);
   fprintf(stderr,"\n");
+  exit(1);
+}
+
+extern int do_warning;
+
+void warning(int n, const char *format, ...)
+{
+  va_list ap;
+  va_start(ap, format);
+  if (do_warning)
+  {
+    fprintf(stderr,"Warning:");
+    msgprefix(n);
+    vfprintf(stderr,format,ap);
+    fprintf(stderr,"\n");
+    printnode(stderr,n);
+    fprintf(stderr,"\n");
+  }
   va_end(ap);
 }
 
-int listtostack(int n, int f, int *stack, int nbmax, const char *error,int nerr)
+int listtostack(int n, Ffunc f, int *stack, int nbmax, const char *error,int nerr)
 {
   int x,i,nb;
   if (n==-1) return 0;
@@ -67,7 +93,7 @@ int listcalltostack(int n, int fnum, int *stack, int nbmax, const char *error,in
   {
     int xx=tree[x].x;
     int xy=tree[x].y;
-    if (tree[x].f!=Fentryfunc || xx!=fnum) break;
+    if (tree[x].f!=Ffunction || xx!=fnum) break;
     x=tree[xy].x;
     i++;
     if (i>=nbmax)
@@ -83,7 +109,7 @@ int listcalltostack(int n, int fnum, int *stack, int nbmax, const char *error,in
   stack[0]=x;
   return nb;
 }
-int listtostackparent(int n, int f, int *stack, int nbmax, const char *error,int nerr)
+int listtostackparent(int n, Ffunc f, int *stack, int nbmax, const char *error,int nerr)
 {
   int x,i,nb;
   if (n==-1) return 0;
@@ -95,6 +121,7 @@ int listtostackparent(int n, int f, int *stack, int nbmax, const char *error,int
   for(x=n;i>=0;stack[i]=x,x=tree[x].x,i--);
   return nb;
 }
+
 int genlistargs(int n,int *stack,int min,int max)
 {
   const char *name=value[tree[n].x].val.str;
@@ -105,47 +132,108 @@ int genlistargs(int n,int *stack,int min,int max)
   return nb;
 }
 
-PPproto parseproto(char const **q, char *c)
+void killlistarg(int n, int a)
+{
+  int x=tree[n].y, p;
+  if (x==a)
+    tree[n].y = GNOARG;
+  else
+  {
+    for(p=n; tree[x].f==Flistarg; p=x, x=tree[x].x)
+      if (tree[x].y==a)
+        tree[x].y=GNOARG;
+    if (x==a) tree[p].x=GNOARG;
+  }
+}
+
+int genlistcats(int y,int *stack,int max)
+{
+  return listcalltostack(y,OPcat,stack,max,"concatenation",y);
+}
+
+PPproto
+parseproto(char const **q, char *c)
 {
   char  const *p=*q;
   long i;
   switch(*p)
   {
   case 0:
+  case '\n':
     return PPend;
   case 'D':
     switch(p[1])
     {
     case 0:
-      die(-1,"incomplete prototype");
+      die(-1,"function has incomplete prototype");
     case 'G':
     case '&':
+    case 'W':
     case 'V':
+    case 'r':
+    case 's':
     case 'I':
+    case 'E':
     case 'n':
+    case 'P':
       *c=p[1];
       *q=p+2;
       return PPdefault;
     default:
       for(i=0;*p && i<2;p++) i+=*p==',';
       if (i<2)
-        die(-1,"incomplete prototype");
+        die(-1,"function has incomplete prototype");
       *c=p[-2];
       *q=p;
       return PPdefaultmulti;
     }
     break;
-  case '=':
+  case 'C':
+  case 'p':
+  case 'P':
+  case 'f':
     *c=*p;
     *q=p+1;
-    return PPsep;
-  case 's':
-    if (p[1]=='*')
-      p++;
-  default:/*fall through*/
-    *c=*p;
+    return PPauto;
+  case '&':
+    *c='*';
     *q=p+1;
     return PPstd;
-    break;
+  case 'V':
+    if (p[1]=='=')
+    {
+      if (p[2]!='G')
+        die(-1,"function prototype is not supported");
+      *c='=';
+      p+=2;
+    }
+    else
+      *c=*p;
+    *q=p+1;
+    return PPstd;
+  case '*':
+    *c=*p++;
+    *q=p+1;
+    return PPstar;
+  case 's':
+    if (p[1]=='*')
+    {
+      *c=*p++;
+      *q=p+1;
+      return PPstar;
+    }
+    /*fall through*/
   }
+  *c=*p;
+  *q=p+1;
+  return PPstd;
+}
+
+char *xstrndup(const char *s, size_t n)
+{
+  char *t=(char*)malloc(n+1);
+  size_t i;
+  for(i=0;i<n;i++) t[i]=s[i];
+  t[i]=0;
+  return t;
 }
